// Copyright (c) 2014-2016 Dr. Colin Hirsch and Daniel Frey
// Please see LICENSE for license or visit https://github.com/ColinH/PEGTL/

#ifndef PEGTL_HH
#define PEGTL_HH

#include "pegtl/parse.hh"
#include "pegtl/rules.hh"
#include "pegtl/ascii.hh"
#include "pegtl/utf8.hh"
#include "pegtl/utf16.hh"
#include "pegtl/utf32.hh"

// The following files can be included whenever needed; they
// are not included by default because they include <iostream>.
// #include "pegtl/trace.hh"
// #include "pegtl/analyze.hh"

#include "pegtl/string_parser.hh"
#include "pegtl/file_parser.hh"

#endif
