// Copyright (c) 2014-2015 Dr. Colin Hirsch and Daniel Frey
// Please see LICENSE for license or visit https://github.com/ColinH/PEGTL/

#ifndef PEGTL_APPLY_MODE_HH
#define PEGTL_APPLY_MODE_HH

namespace pegtl
{
   enum class apply_mode : bool
   {
      ACTION = true,
      NOTHING = false
   };

} // pegtl

#endif
